//
//  ScratchUIView.swift
//  ScratchCard
//
//  Created by JoeJoe on 2016/4/15.
//  Copyright © 2016年 JoeJoe. All rights reserved.
//

import Foundation
import UIKit

var uiScratchWidth: CGFloat!

@objc public protocol ScratchUIViewDelegate: AnyObject {
    @objc optional func scratchBegan(_ view: ScratchUIView)
    @objc optional func scratchMoved(_ view: ScratchUIView)
    @objc optional func scratchEnded(_ view: ScratchUIView)
}

open class ScratchUIView: UIView, ScratchViewDelegate {

    open weak var delegate: ScratchUIViewDelegate!
    public var scratchView: ScratchView!
    var maskImage: UIImageView!
    var maskPath: String!
    var coupponPath: String!
    open var scratchPosition: CGPoint!
    var couponImage: UIImage?
    var maskUIImage: UIImage?
    override init(frame: CGRect) {
        super.init(frame: frame)
        self.Init()
    }

    open func getScratchPercent() -> Double {
        return scratchView.getAlphaPixelPercent()
    }

    public init(frame: CGRect, Coupon: String, MaskImage: String, ScratchWidth: CGFloat) {
        super.init(frame: frame)
        coupponPath = Coupon
        maskPath = MaskImage
        uiScratchWidth = ScratchWidth
        self.Init()
    }

    public init(frame: CGRect, couponImage: UIImage, maskImage: UIImage, scratchWidth: CGFloat) {
        super.init(frame: frame)
        self.couponImage = couponImage
        self.maskUIImage = maskImage
        uiScratchWidth = scratchWidth
        self.Init()
    }

    required public init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        self.InitXib()
    }

    fileprivate func Init() {
        if let mask = self.maskUIImage {
            maskImage = UIImageView(image: mask)
        } else {
            maskImage = UIImageView(image: UIImage(named: maskPath))
        }

        if let img =  self.couponImage {
            scratchView = ScratchView(frame: self.frame, img: img, width: uiScratchWidth)
        } else {
            scratchView = ScratchView(frame: self.frame, CouponImage: coupponPath, ScratchWidth: uiScratchWidth)
        }

        maskImage.frame = CGRect(x: 0, y: 0, width: self.frame.width, height: self.frame.height)
        scratchView.frame = CGRect(x: 0, y: 0, width: self.frame.width, height: self.frame.height)
        scratchView.delegate = self
        self.addSubview(maskImage)
        self.addSubview(scratchView)
        self.bringSubviewToFront(scratchView)

    }

    internal func began(_ view: ScratchView) {
        if self.delegate != nil {
            guard self.delegate.scratchBegan != nil else {
                return
            }
            if view.position.x >= 0 && view.position.x <= view.frame.width && view.position.y >= 0 && view.position.y <= view.frame.height {
                scratchPosition = view.position
            }
            self.delegate.scratchBegan!(self)
        }
    }

    internal func moved(_ view: ScratchView) {
        if self.delegate != nil {
            guard self.delegate.scratchMoved != nil else {
                return
            }
            if view.position.x >= 0 && view.position.x <= view.frame.width && view.position.y >= 0 && view.position.y <= view.frame.height {
                scratchPosition = view.position
            }
            self.delegate.scratchMoved!(self)
        }
    }

    internal func ended(_ view: ScratchView) {
        if self.delegate != nil {
            guard self.delegate.scratchEnded != nil else {
                return
            }
            if view.position.x >= 0 && view.position.x <= view.frame.width && view.position.y >= 0 && view.position.y <= view.frame.height {
                scratchPosition = view.position
            }
            self.delegate.scratchEnded!(self)
        }
    }

    fileprivate func InitXib() {

    }
}

extension UIView {
    func asImage() -> UIImage {
        let renderer = UIGraphicsImageRenderer(bounds: bounds)
        return renderer.image { rendererContext in
            layer.render(in: rendererContext.cgContext)
        }
    }
}
